from .barCodePayment import BarCodePayment
from .nonBarCodePayment import NonBarCodePayment


class DarfBarCodePayment(BarCodePayment):
    pass


class DarfPayment(NonBarCodePayment):

    def setPayment(self, **kwargs):
        self.setSender(kwargs.get("sender"))
        self.setTaxPaymentIdentifier("02")
        self.setRevenueCode(kwargs.get("revenueCode"))
        self.setTaxIdInfo(kwargs.get("taxId"))
        self.setReferenceDate(kwargs.get("referenceDate"))
        self.setReferenceNumber(kwargs.get("referenceNumber"))
        self.setNominalAmount(kwargs.get("amount"))
        self.setFineAmount(kwargs.get("fine"))
        self.setInterestAmount(kwargs.get("interest"))
        self.setTotalAmount(kwargs.get("totalAmount"))
        self.setDueDate(kwargs.get("dueDate"))
        self.setScheduleDate(kwargs.get("scheduleDate"))
        self.setIdentifier(kwargs.get("identifier"))

    def setTaxPaymentIdentifier(self, id):
        self.segmentN.setTaxPaymentIdentifier(id)

    def setRevenueCode(self, code):
        self.segmentN.setRevenueCode(code)

    def setTaxIdInfo(self, taxId):
        taxId = "".join(c for c in taxId if c.isdigit())
        taxIdType = "1" if len(taxId) == 11 else "2"
        self.segmentN.setTaxIdType(taxIdType)
        self.segmentN.setTaxId(taxId)

    def setReferenceDate(self, referenceDate):
        self.segmentN.setReferenceDate(referenceDate)

    def setReferenceNumber(self, referenceNumber):
        self.segmentN.setReferenceNumber(referenceNumber)

    def setNominalAmount(self, amount):
        self.segmentN.setNominalAmount(amount)
        self.amount = int(amount)

    def setInterestAmount(self, interestAmount):
        self.segmentN.setInterestAmount(interestAmount)
        self.additionAmount += int(interestAmount)

    def setFineAmount(self, fineAmount):
        self.segmentN.setFineAmount(fineAmount)
        self.additionAmount += int(fineAmount)

    def setTotalAmount(self, totalAmount):
        self.segmentN.setTotalAmount(totalAmount)
        self.totalAmount = int(totalAmount)

    def setDueDate(self, dueDate):
        self.segmentN.setDueDate(dueDate)

    def setScheduleDate(self, paymentDate):
        self.segmentN.setScheduleDate(paymentDate)



# coding: utf-8

occurrences = {
    "00": "PAGAMENTO EFETUADO",
    "AE": "DATA DE PAGAMENTO ALTERADA",
    "AG": "NÚMERO DO LOTE INVÁLIDO",
    "AH": "NÚMERO SEQUENCIAL DO REGISTRO NO LOTE INVÁLIDO",
    "AI": "PRODUTO DEMONSTRATIVO DE PAGAMENTO NÃO CONTRATADO",
    "AJ": "TIPO DE MOVIMENTO INVÁLIDO",
    "AL": "CÓDIGO DO BANCO FAVORECIDO INVÁLIDO",
    "AM": "AGÊNCIA DO FAVORECIDO INVÁLIDA",
    "AN": "CONTA CORRENTE DO FAVORECIDO INVÁLIDA / CONTA INVESTIMENTO EXTINTA EM 30/04/2011",
    "AO": "NOME DO FAVORECIDO INVÁLIDO",
    "AP": "DATA DE PAGAMENTO / DATA DE VALIDADE / HORA DE LANÇAMENTO / ARRECADAÇÃO / APURAÇÃO INVÁLIDA",
    "AQ": "QUANTIDADE DE REGISTROS MAIOR QUE 999999",
    "AR": "VALOR ARRECADADO / LANÇAMENTO INVÁLIDO",
    "BC": "NOSSO NÚMERO INVÁLIDO",
    "BD": "PAGAMENTO AGENDADO",
    "BE": "PAGAMENTO AGENDADO COM FORMA ALTERADA PARA OP",
    "BI": "CNPJ / CPF DO FAVORECIDO NO SEGMENTOJ-52 ou B INVÁLIDO",
    "BL": "VALOR DA PARCELA INVÁLIDO",
    "CD": "CNPJ / CPF INFORMADO DIVERGENTE DO CADASTRADO",
    "CE": "PAGAMENTO CANCELADO",
    "CF": "VALOR DO DOCUMENTO INVÁLIDO",
    "CG": "VALOR DO ABATIMENTO INVÁLIDO",
    "CH": "VALOR DO DESCONTO INVÁLIDO",
    "CI": "CNPJ / CPF / IDENTIFICADOR / INSCRIÇÃO ESTADUAL / INSCRIÇÃO NO CAD / ICMS INVÁLIDO",
    "CJ": "VALOR DA MULTA INVÁLIDO",
    "CK": "TIPO DE INSCRIÇÃO INVÁLIDA",
    "CL": "VALOR DO INSS INVÁLIDO",
    "CM": "VALOR DO COFINS INVÁLIDO",
    "CN": "CONTA NÃO CADASTRADA",
    "CO": "VALOR DE OUTRAS ENTIDADES INVÁLIDO",
    "CP": "CONFIRMAÇÃO DE OP CUMPRIDA",
    "CQ": "SOMA DAS FATURAS DIFERE DO PAGAMENTO",
    "CR": "VALOR DO CSLL INVÁLIDO",
    "CS": "DATA DE VENCIMENTO DA FATURA INVÁLIDA",
    "DA": "NÚMERO DE DEPEND. SALÁRIO FAMILIA INVALIDO",
    "DB": "NÚMERO DE HORAS SEMANAIS INVÁLIDO",
    "DC": "SALÁRIO DE CONTRIBUIÇÃO INSS INVÁLIDO",
    "DD": "SALÁRIO DE CONTRIBUIÇÃO FGTS INVÁLIDO",
    "DE": "VALOR TOTAL DOS PROVENTOS INVÁLIDO",
    "DF": "VALOR TOTAL DOS DESCONTOS INVÁLIDO",
    "DG": "VALOR LÍQUIDO NÃO NUMÉRICO",
    "DH": "VALOR LIQ. INFORMADO DIFERE DO CALCULADO",
    "DI": "VALOR DO SALÁRIO-BASE INVÁLIDO",
    "DJ": "BASE DE CÁLCULO IRRF INVÁLIDA",
    "DK": "BASE DE CÁLCULO FGTS INVÁLIDA",
    "DL": "FORMA DE PAGAMENTO INCOMPATÍVEL COM HOLERITE",
    "DM": "E-MAIL DO FAVORECIDO INVÁLIDO",
    "DV": "DOC / TED DEVOLVIDO PELO BANCO FAVORECIDO",
    "D0": "FINALIDADE DO HOLERITE INVÁLIDA",
    "D1": "MÊS DE COMPETENCIA DO HOLERITE INVÁLIDA",
    "D2": "DIA DA COMPETENCIA DO HOLETITE INVÁLIDA",
    "D3": "CENTRO DE CUSTO INVÁLIDO",
    "D4": "CAMPO NUMÉRICO DA FUNCIONAL INVÁLIDO",
    "D5": "DATA INÍCIO DE FÉRIAS NÃO NUMÉRICA",
    "D6": "DATA INÍCIO DE FÉRIAS INCONSISTENTE",
    "D7": "DATA FIM DE FÉRIAS NÃO NUMÉRICO",
    "D8": "DATA FIM DE FÉRIAS INCONSISTENTE",
    "D9": "NÚMERO DE DEPENDENTES IR INVÁLIDO",
    "EM": "CONFIRMAÇÃO DE OP EMITIDA",
    "EX": "DEVOLUÇÃO DE OP NÃO SACADA PELO FAVORECIDO",
    "E0": "TIPO DE MOVIMENTO HOLERITE INVÁLIDO",
    "E1": "VALOR 01 DO HOLERITE / INFORME INVÁLIDO",
    "E2": "VALOR 02 DO HOLERITE / INFORME INVÁLIDO",
    "E3": "VALOR 03 DO HOLERITE / INFORME INVÁLIDO",
    "E4": "VALOR 04 DO HOLERITE / INFORME INVÁLIDO",
    "FC": "PAGAMENTO EFETUADO ATRAVÉS DE FINANCIAMENTO COMPROR",
    "FD": "PAGAMENTO EFETUADO ATRAVÉS DE FINANCIAMENTO DESCOMPROR",
    "HA": "ERRO NO HEADER DE ARQUIVO",
    "HM": "ERRO NO HEADER DE LOTE",
    "IB": "VALOR E/OU DATA DO DOCUMENTO INVÁLIDO",
    "IC": "VALOR DO ABATIMENTO INVÁLIDO",
    "ID": "VALOR DO DESCONTO INVÁLIDO",
    "IE": "VALOR DA MORA INVÁLIDO",
    "IF": "VALOR DA MULTA INVÁLIDO",
    "IG": "VALOR DA DEDUÇÃO INVÁLIDO",
    "IH": "VALOR DO ACRÉSCIMO INVÁLIDO",
    "II": "DATA DE VENCIMENTO INVÁLIDA",
    "IJ": "COMPETÊNCIA / PERÍODO REFERÊNCIA / PARCELA INVÁLIDA",
    "IK": "TRIBUTO NÃO LIQUIDÁVEL VIA SISPAG OU NÃO CONVENIADO COM ITAÚ",
    "IL": "CÓDIGO DE PAGAMENTO / EMPRESA /RECEITA INVÁLIDO",
    "IM": "TIPO X FORMA NÃO COMPATÍVEL",
    "IN": "BANCO/AGENCIA NÃO CADASTRADOS",
    "IO": "DAC / VALOR / COMPETÊNCIA / IDENTIFICADOR DO LACRE INVÁLIDO",
    "IP": "DAC DO CÓDIGO DE BARRAS INVÁLIDO",
    "IQ": "DÍVIDA ATIVA OU NÚMERO DE ETIQUETA INVÁLIDO",
    "IR": "PAGAMENTO ALTERADO",
    "IS": "CONCESSIONÁRIA NÃO CONVENIADA COM ITAÚ",
    "IT": "VALOR DO TRIBUTO INVÁLIDO",
    "IU": "VALOR DA RECEITA BRUTA ACUMULADA INVÁLIDO",
    "IV": "NÚMERO DO DOCUMENTO ORIGEM / REFERÊNCIA INVÁLIDO",
    "IX": "CÓDIGO DO PRODUTO INVÁLIDO",
    "LA": "DATA DE PAGAMENTO DE UM LOTE ALTERADA",
    "LC": "LOTE DE PAGAMENTOS CANCELADO",
    "NA": "PAGAMENTO CANCELADO POR FALTA DE AUTORIZAÇÃO",
    "NB": "IDENTIFICAÇÃO DO TRIBUTO INVÁLIDA",
    "NC": "EXERCÍCIO (ANO BASE) INVÁLIDO",
    "ND": "CÓDIGO RENAVAM NÃO ENCONTRADO/INVÁLIDO",
    "NE": "UF INVÁLIDA",
    "NF": "CÓDIGO DO MUNICÍPIO INVÁLIDO",
    "NG": "PLACA INVÁLIDA",
    "NH": "OPÇÃO/PARCELA DE PAGAMENTO INVÁLIDA",
    "NI": "TRIBUTO JÁ FOI PAGO OU ESTÁ VENCIDO",
    "NR": "OPERAÇÃO NÃO REALIZADA RJ REGISTRO REJEITADO",
    "PD": "AQUISIÇÃO CONFIRMADA (EQUIVALE A OCORRÊNCIA 02 NO LAYOUT DE DISCO SACADO)",
    "RJ": "REGISTRO REJEITADO",
    "RS": "PAGAMENTO DISPONÍVEL PARA ANTECIPAÇÃO NO RISCO SACADO – MODALIDADE RISCO SACADO PÓS AUTORIZADO",
    "SS": "PAGAMENTO CANCELADO POR INSUFICIÊNCIA DE SALDO/LIMITE DIÁRIO DE PAGTO",
    "TA": "LOTE NÃO ACEITO - TOTAIS DO LOTE COM DIFERENÇA",
    "TI": "TITULARIDADE INVÁLIDA",
    "X1": "FORMA INCOMPATÍVEL COM LAYOUT 010",
    "X2": "NÚMERO DA NOTA FISCAL INVÁLIDO",
    "X3": "IDENTIFICADOR DE NF/CNPJ INVÁLIDO",
    "X4": "FORMA 32 INVÁLIDA",
}


from .file.file import File
from .slip.slip import Slip
from .slip.segmentQ import SegmentQ
from .slip.segmentP import SegmentP
from .slip.segmentR import SegmentR
from .result.parser import SlipParser


from ....row import Row
from ....characterType import numeric, alphaNumeric


class SegmentQ:

    def __init__(self):
        self.content = "0000000000000  000000000000000000                                                                                               00000000                 0000000000000000                                        000                            "
        self.defaultValues()

    def defaultValues(self):
        structs = [
            ( 3,  7, 4,      numeric, "1"),
            ( 7,  8, 1,      numeric, "3"),
            (13, 14, 1, alphaNumeric, "Q"),
            (15, 17, 2,      numeric, "01"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setCancel(self):
        structs = [
            (15, 17, 2, numeric, "02"),                                               # Indica Baixa do boleto
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0, 3, 3, numeric, bank.bankId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index)                                                # Indica index do lote
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPayer(self, user):
        structs = [
            (17,  18,  1,      numeric, "1" if len(user.identifier) == 11 else "2"),  # 1 - CPF/ 2 - CNPJ
            (18,  33, 15,      numeric, user.identifier),                             # CPF/CNPJ do Pagador
            (33,  63, 30, alphaNumeric, user.name)                                    # Nome do Pagador
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPayerAddress(self, address):
        structs = [
            ( 73, 113, 40, alphaNumeric, "%s %s" % (address.streetLine1, address.streetLine2)),
            (113, 128, 15, alphaNumeric, address.district),
            (128, 136,  8,      numeric, address.zipCode),
            (136, 151, 15, alphaNumeric, address.city),
            (151, 153,  2, alphaNumeric, address.stateCode),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setGuarantor(self, user):
        structs = [
            (153, 154,  1,      numeric, "1" if len(user.identifier) == 11 else "2"), # 1 - CPF/ 2 - CNPJ
            (154, 169, 15,      numeric, user.identifier),                            # CPF/CNPJ do Sacador Avalista
            (169, 199, 30, alphaNumeric, user.name)                                   # Nome do Sacador Avalista
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from datetime import datetime
from .header import Header
from .trailer import Trailer
from ....libs.fileUtils import FileUtils


class File:

    def __init__(self):
        self.header = Header()
        self.lots = []
        self.trailer = Trailer()

    def addLot(self, lot):
        lot.setLotNumber(len(self.lots) + 1)
        self.lots.append(lot)

    def toString(self, currentDatetime=None):
        lotsToString = "\r\n".join([lot.toString() for lot in self.lots])
        self.header.setGeneratedFileDate(currentDatetime or datetime.now())
        self.trailer.setNumberOfLotsAndRegisters(
            num=len(self.lots),
            sum=2 + self._countRegistersInLots()
        )
        return "%s\r\n%s\r\n%s\r\n" % (
            self.header.content,
            lotsToString,
            self.trailer.content
        )

    def setSender(self, sender):
        self.header.setSender(sender)
        self.header.setSenderBank(sender.bank)
        self.trailer.setSenderBank(sender.bank)

    def output(self, fileName, path="/../", content=None, currentDatetime=None):
        file = FileUtils.create(name=fileName, path=path)
        file.write(self.toString(currentDatetime or datetime.now()) if not content else content)
        file.close()

    def _countRegistersInLots(self):
        count = 0
        for lot in self.lots:
            count += lot.count
        return count



from .nonBarCodePayment import NonBarCodePayment


class FgtsPayment(NonBarCodePayment):

    def setPayment(self, **kwargs):
        self.setSender(kwargs.get("sender"))
        self.setTaxPaymentIdentifier("11")
        self.setRevenueCode("0000")
        self.setTaxIdInfo(kwargs.get("taxId"))
        self.setFgtsScheduleDate(kwargs.get("scheduleDate"))
        self.setIdentifier(kwargs.get("identifier"))
        self.setBarCode(kwargs.get("barCode"))
        self.setFgtsAmount(kwargs.get("amount"))
        self.setFgtsNullValues()

    def setTaxPaymentIdentifier(self, id):
        self.segmentN.setTaxPaymentIdentifier(id)

    def setRevenueCode(self, code):
        self.segmentN.setRevenueCode(code)

    def setTaxIdInfo(self, taxId):
        taxId = "".join(c for c in taxId if c.isdigit())
        taxIdType = "1" if len(taxId) == 14 else "2"
        self.segmentN.setTaxIdType(taxIdType)
        self.segmentN.setTaxId(taxId)

    def setReferenceDate(self, referenceDate):
        self.segmentN.setReferenceDate(referenceDate)

    def setReferenceNumber(self, referenceNumber):
        self.segmentN.setReferenceNumber(referenceNumber)

    def setFgtsAmount(self, amount):
        self.segmentN.setFgtsAmount(amount)
        self.amount = int(amount)
        self.totalAmount = int(amount)

    def setDueDate(self, dueDate):
        self.segmentN.setDueDate(dueDate)

    def setFgtsScheduleDate(self, paymentDate):
        self.segmentN.setFgtsScheduleDate(paymentDate)

    def setBarCode(self, barCode):
        self.segmentN.setBarCode(barCode)

    def setFgtsNullValues(self):
        self.segmentN.setFgtsNullValues()



from ....row import Row
from ....characterType import numeric, alphaNumeric


class HeaderLot:

    def __init__(self):
        self.content = "00000000 0000000 0000000000000000                    00000 000000000000 0                                                                                                              000000000000000000000000                                 "
        self.defaultValues()

    def defaultValues(self):
        structs = [
            (  3,   7, 4,      numeric, "1"),
            (  7,   8, 1,      numeric, "1"),
            (  8,   9, 1, alphaNumeric, "R"),
            (  9,  11, 2,      numeric, "01"),
            ( 13,  16, 3,      numeric, "030"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setGeneratedFileDate(self, datetime):
        structs = [
            (191, 199, 8, numeric, datetime.strftime("%d%m%Y")),   # Data de gravacao
            (199, 207, 8, numeric, datetime.strftime("%d%m%Y")),   # Data de credito
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSender(self, user):
        structs = [
            (17,  18,  1,      numeric, "1" if len(user.identifier) == 11 else "2"),
            (18,  33, 15,      numeric, user.identifier),
            (73, 103, 30, alphaNumeric, user.name)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (  0,   3,  3,      numeric, bank.bankId),
            ( 54,  58,  4,      numeric, bank.branchCode),
            ( 59,  71, 12,      numeric, bank.accountNumber),
            ( 72,  73,  1,      numeric, bank.accountVerifier)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (3, 7, 4, numeric, index)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)


from .segmentO import SegmentO


class BarCodePayment:

    def __init__(self):
        self.segmentO = SegmentO()
        self.amount = 0

    def setPayment(self, **kwargs):
        self.setSender(kwargs.get("sender"))
        self.setScheduleDate(kwargs.get("scheduleDate"))
        self.setIdentifier(kwargs.get("identifier"))
        self.setLineNumber(kwargs.get("lineNumber"))
        self.setDueDate(kwargs.get("scheduleDate"))

    def toString(self):
        return "\r\n".join((
            self.segmentO.content,
        ))

    def amountInCents(self):
        return self.amount

    def setSender(self, user):
        self.segmentO.setSenderBank(user.bank)

    def setIdentifier(self, identifier):
        self.segmentO.setIdentifier(identifier)

    def setLineNumber(self, lineNumber):
        self.segmentO.setLineNumber(lineNumber)
        self.amount = int(lineNumber.amount)

    def setScheduleDate(self, paymentDate):
        self.segmentO.setScheduleDate(paymentDate)

    def setDueDate(self, dueDate):
        self.segmentO.setDueDate(dueDate)

    def setPositionInLot(self, index):
        self.segmentO.setPositionInLot(index)

    def setLot(self, lot):
        self.segmentO.setLot(lot)



from febraban.cnab240.libs.enum import Enum


class BarcodePayment(Enum):

    utility = "utility"
    iss = "iss"
    das = "das"
    gps = "gps"
    darf = "darf"
    fgts = "fgts"


class NonBarcodeTaxPayment(Enum):

    gps = "gps-n"
    darf = "darf-n"





class Enum:

    @classmethod
    def values(cls):
        return [getattr(cls, a) for a in dir(cls) if not a.startswith('__') and not callable(getattr(cls, a))]

    @classmethod
    def isValid(cls, enum):
        return enum in cls.values()



from .file.file import File
from .result.parser import PaymentParser
from .payment.transfer import Transfer
from .payment.barCodePayment import BarCodePayment
from .payment.nonBarCodePayment import NonBarCodePayment
from .payment.issPayment import IssPayment
from .payment.dasPayment import DasPayment
from .payment.fgtsPayment import FgtsPayment
from .payment.darfPayment import DarfPayment, DarfBarCodePayment
from .payment.gpsPayment import GpsBarCodePayment
from .payment.chargePayment import ChargePayment
from .payment.utilityPayment import UtilityPayment



#!/usr/bin/python
# -*- coding: latin-1 -*-

errors03 = {
    "03": "AG. COBRADORA: NÃO FOI POSSÍVEL ATRIBUIR A AGÊNCIA PELO CEP OU CEP INVÁLIDO",
    "04": "ESTADO: SIGLA DO ESTADO INVÁLIDA",
    "05": "DATA VENCIMENTO: PRAZO DA OPERAÇÃO MENOR QUE PRAZO MÍNIMO OU MAIOR QUE O MÁXIMO",
    "08": "NOME DO PAGADOR: NÃO INFORMADO OU DESLOCADO",
    "09": "AGÊNCIA/CONTA: AGÊNCIA ENCERRADA",
    "10": "LOGRADOURO: NÃO INFORMADO OU DESLOCADO",
    "11": "CEP: CEP NÃO NUMÉRICO",
    "12": "SACADOR AVALISTA: NOME NÃO INFORMADO OU DESLOCADO",
    "13": "ESTADO/CEP: CEP INCOMPATÍVEL COM A SIGLA DO ESTADO",
    "14": "NOSSO NÚMERO: NOSSO NÚMERO JÁ REGISTRADO NO CADASTRO DO BANCO OU FORA DA FAIXA",
    "15": "NOSSO NÚMERO: NOSSO NÚMERO EM DUPLICIDADE NO MESMO MOVIMENTO",
    "18": "DATA DE ENTRADA: DATA DE ENTRADA INVÁLIDA PARA OPERAR COM ESTA CARTEIRA",
    "19": "OCORRÊNCIA: OCORRÊNCIA INVÁLIDA",
    "21": "AG. COBRADORA: CARTEIRA NÃO ACEITA DEPOSITÁRIA CORRESPONDENTE/ ESTADO DA AGÊNCIA DIFERENTE DO ESTADO DO PAGADOR/ AG. COBRADORA NÃO CONSTA NO CADASTRO OU ENCERRANDO",
    "22": "CARTEIRA: CARTEIRA NÃO PERMITIDA (NECESSÁRIO CADASTRAR FAIXA LIVRE)",
    "27": "CNPJ INAPTO: CNPJ DO BENEFICIÁRIO INAPTO DEVOLUÇÃO DE TÍTULO EM GARANTIA",
    "29": "CÓDIGO EMPRESA: CATEGORIA DA CONTA INVÁLIDA",
    "31": "AGÊNCIA/CONTA: CONTA NÃO TEM PERMISSÃO PARA PROTESTAR (CONTATE SEU GERENTE)",
    "35": "VALOR DO IOF: IOF MAIOR QUE 5%",
    "36": "QTDADE DE MOEDA: QUANTIDADE DE MOEDA INCOMPATÍVEL COM VALOR DO TÍTULO",
    "37": "CNPJ/CPF DO PAGADOR: NÃO NUMÉRICO OU IGUAL A ZEROS",
    "42": "NOSSO NÚMERO: NOSSO NÚMERO FORA DA FAIXA",
    "52": "AG. COBRADORA: EMPRESA NÃO ACEITA BANCO CORRESPONDENTE",
    "53": "AG. COBRADORA: EMPRESA NÃO ACEITA BANCO CORRESPONDENTE - COBRANÇA MENSAGEM",
    "54": "DATA DE VENCTO: BANCO CORRESPONDENTE – TÍTULO COM VENCIMENTO INFERIOR A 15 DIAS",
    "55": "DEP./BCO. CORRESP.: CEP NÃO PERTENCE A DEPOSITÁRIA INFORMADA",
    "56": "DT. VCTO./BCO. CORESP.: VENCTO. SUPERIOR A 180 DIAS DA DATA DE ENTRADA",
    "57": "DATA DE VENCIMENTO: CEP SÓ DEPOSITÁRIA BCO. DO BRASIL COM VENCTO. INFERIOR A 8 DIAS",
    "60": "ABATIMENTO: VALOR DO ABATIMENTO INVÁLIDO",
    "61": "JUROS DE MORA: JUROS DE MORA MAIOR QUE O PERMITIDO",
    "62": "DESCONTO: VALOR DO DESCONTO MAIOR QUE O VALOR DO TÍTULO",
    "63": "DESCONTO DE ANTECIPAÇÃO: VALOR DA IMPORTÂNCIA POR DIA DE DESCONTO (IDD) NÃO PERMITIDO",
    "64": "EMISSÃO DO TÍTULO: DATA DE EMISSÃO DO TÍTULO INVÁLIDA (VENDOR)",
    "65": "TAXA FINANCTO.: TAXA INVÁLIDA (VENDOR)",
    "66": "DATA DE VENCTO.: INVALIDA/FORA DE PRAZO DE OPERAÇÃO (MÍNIMO OU MÁXIMO)",
    "67": "VALOR/QTIDADE.: VALOR DO TÍTULO/QUANTIDADE DE MOEDA INVÁLIDO",
    "68": "CARTEIRA: CARTEIRA INVÁLIDA OU NÃO CADASTRADA NO INTERCÂMBIO DA COBRANÇA",
    "98": "FLASH INVÁLIDO: REGISTRO MENSAGEM SEM FLASH CADASTRADO OU FLASH INFORMADO DIFERENTE DO CADASTRADO",
    "99": "FLASH INVÁLIDO: CONTA DE COBRANÇA COM FLASH CADASTRADO E SEM REGISTRO DE MENSAGEM CORRESPONDENTE",
}

errors17 = {
    "02": "AGÊNCIA COBRADORA INVÁLIDA OU COM O MESMO CONTEÚDO",
    "04": "SIGLA DO ESTADO INVÁLIDA",
    "05": "DATA DE VENCIMENTO INVÁLIDA OU COM O MESMO CONTEÚDO",
    "06": "VALOR DO TÍTULO COM OUTRA ALTERAÇÃO SIMULTÂNEA",
    "08": "NOME DO PAGADOR COM O MESMO CONTEÚDO",
    "11": "CEP INVÁLIDO",
    "12": "NÚMERO INSCRIÇÃO INVÁLIDO DO SACADOR AVALISTA",
    "13": "SEU NÚMERO COM O MESMO CONTEÚDO",
    "21": "AGÊNCIA COBRADORA NÃO CONSTA NO CADASTRO DE DEPOSITÁRIA OU EM ENCERRAMENTO",
    "42": "ALTERAÇÃO INVÁLIDA PARA TÍTULO VENCIDO",
    "43": "ALTERAÇÃO BLOQUEADA – VENCIMENTO JÁ ALTERADO",
    "53": "INSTRUÇÃO COM O MESMO CONTEÚDO",
    "54": "DATA VENCIMENTO PARA BANCOS CORRESPONDENTES INFERIOR AO ACEITO PELO BANCO",
    "55": "ALTERAÇÕES IGUAIS PARA O MESMO CONTROLE (AGÊNCIA/CONTA/CARTEIRA/NOSSO NÚMERO)",
    "60": "VALOR DE IOF – ALTERAÇÃO NÃO PERMITIDA PARA CARTEIRAS DE N.S. – MOEDA VARIÁVEL",
    "61": "TÍTULO JÁ BAIXADO OU LIQUIDADO OU NÃO EXISTE TÍTULO CORRESPONDENTE NO SISTEMA",
    "66": "ALTERAÇÃO NÃO PERMITIDA PARA CARTEIRAS DE NOTAS DE SEGUROS – MOEDA VARIÁVEL",
    "67": "NOME INVÁLIDO DO SACADOR AVALISTA",
    "72": "ENDEREÇO INVÁLIDO – SACADOR AVALISTA",
    "73": "BAIRRO INVÁLIDO – SACADOR AVALISTA",
    "74": "CIDADE INVÁLIDA – SACADOR AVALISTA",
    "75": "SIGLA ESTADO INVÁLIDO – SACADOR AVALISTA",
    "76": "CEP INVÁLIDO – SACADOR AVALISTA",
    "81": "ALTERAÇÃO BLOQUEADA - TÍTULO COM NEGATIVAÇÃO EXPRESSA OU PROTESTO",
}

errors16 = {
    "01": "INSTRUÇÃO/OCORRÊNCIA NÃO EXISTENTE",
    "03": "CONTA NÃO TEM PERMISSÃO PARA PROTESTAR (CONTATE SEU GERENTE) 06 NOSSO NÚMERO IGUAL A ZEROS",
    "09": "CNPJ/CPF DO SACADOR/AVALISTA INVÁLIDO",
    "14": "REGISTRO EM DUPLICIDADE",
    "15": "CNPJ/CPF INFORMADO SEM NOME DO SACADOR/AVALISTA",
    "19": "VALOR DO ABATIMENTO MAIOR QUE 90% DO VALOR DO TÍTULO",
    "20": "EXISTE SUSTACAO DE PROTESTO PENDENTE PARA O TITULO",
    "21": "TÍTULO NÃO REGISTRADO NO SISTEMA",
    "22": "TÍTULO BAIXADO OU LIQUIDADO",
    "23": "INSTRUÇÃO NÃO ACEITA",
    "24": "INSTRUÇÃO INCOMPATÍVEL - EXISTE INSTRUÇÃO DE PROTESTO PARA O TÍTULO",
    "25": "INSTRUÇÃO INCOMPATÍVEL - NÃO EXISTE INSTRUÇÃO DE PROTESTO PARA O TÍTULO",
    "26": "INSTRUÇÃO NÃO ACEITA POR JÁ TER SIDO EMITIDA A ORDEM DE PROTESTO AO CARTÓRIO",
    "27": "INSTRUÇÃO NÃO ACEITA POR NÃO TER SIDO EMITIDA A ORDEM DE PROTESTO AO CARTÓRIO",
    "28": "JÁ EXISTE UMA MESMA INSTRUÇÃO CADASTRADA ANTERIORMENTE PARA O TÍTULO",
    "29": "VALOR LÍQUIDO + VALOR DO ABATIMENTO DIFERENTE DO VALOR DO TÍTULO REGISTRADO",
    "30": "EXISTE UMA INSTRUÇÃO DE NÃO PROTESTAR ATIVA PARA O TÍTULO",
    "31": "EXISTE UMA OCORRÊNCIA DO PAGADOR QUE BLOQUEIA A INSTRUÇÃO",
    "32": "DEPOSITÁRIA DO TÍTULO = 9999 OU CARTEIRA NÃO ACEITA PROTESTO",
    "33": "ALTERAÇÃO DE VENCIMENTO IGUAL À REGISTRADA NO SISTEMA OU QUE TORNA O TÍTULO VENCIDO",
    "34": "INSTRUÇÃO DE EMISSÃO DE AVISO DE COBRANÇA PARA TÍTULO VENCIDO ANTES DO VENCIMENTO",
    "35": "SOLICITAÇÃO DE CANCELAMENTO DE INSTRUÇÃO INEXISTENTE",
    "36": "TÍTULO SOFRENDO ALTERAÇÃO DE CONTROLE (AGÊNCIA/CONTA/CARTEIRA/NOSSO NÚMERO)",
    "37": "INSTRUÇÃO NÃO PERMITIDA PARA A CARTEIRA",
    "40": "INSTRUÇÃO INCOMPATÍVEL – NÃO EXISTE INSTRUÇÃO DE NEGATIVAÇÃO EXPRESSA PARA O TÍTULO",
    "41": "INSTRUÇÃO NÃO PERMITIDA – TÍTULO JÁ ENVIADO PARA NEGATIVAÇÃO EXPRESSA",
    "42": "INSTRUÇÃO NÃO PERMITIDA – TÍTULO COM NEGATIVAÇÃO EXPRESSA CONCLUÍDA",
    "43": "PRAZO INVÁLIDO PARA NEGATIVAÇÃO – MÍNIMO: 02 DIAS CORRIDOS APÓS O VENCIMENTO",
    "45": "INSTRUÇÃO INCOMPATÍVEL PARA O MESMO TÍTULO NESTA DATA",
    "47": "INSTRUÇÃO NÃO PERMITIDA – ESPÉCIE INVÁLIDA",
    "48": "DADOS DO PAGADOR INVÁLIDOS (CPF / CNPJ / NOME)",
    "49": "DADOS DO ENDEREÇO DO PAGADOR INVÁLIDOS",
    "50": "DATA DE EMISSÃO DO TÍTULO INVÁLIDA",
    "51": "INSTRUÇÃO NÃO PERMITIDA – TÍTULO COM NEGATIVAÇÃO EXPRESSA AGENDADA",
}

errors15 = {
    "04": "NOSSO NÚMERO EM DUPLICIDADE NUM MESMO MOVIMENTO",
    "05": "SOLICITAÇÃO DE BAIXA PARA TÍTULO JÁ BAIXADO OU LIQUIDADO",
    "06": "SOLICITAÇÃO DE BAIXA PARA TÍTULO NÃO REGISTRADO NO SISTEMA",
    "07": "COBRANÇA PRAZO CURTO - SOLICITAÇÃO DE BAIXA P/ TÍTULO NÃO REGISTRADO NO SISTEMA",
    "08": "SOLICITAÇÃO DE BAIXA PARA TÍTULO EM FLOATING",
}

errors18 = {
    "16": "ABATIMENTO/ALTERAÇÃO DO VALOR DO TÍTULO OU SOLICITAÇÃO DE BAIXA BLOQUEADOS",
    "40": "NÃO APROVADA DEVIDO AO IMPACTO NA ELEGIBILIDADE DE GARANTIAS",
    "41": "AUTOMATICAMENTE REJEITADA",
    "42": "CONFIRMA RECEBIMENTO DE INSTRUÇÃO – PENDENTE DE ANÁLISE",
}



from ....row import Row
from ....characterType import numeric, alphaNumeric


class SegmentR:

    def __init__(self):
        self.content = "0000000000000  000000000000000000000000000000000000000000000000000                                                                                                                                     0000000000000000 000000000000  0         "
        self.defaultValues()

    def defaultValues(self):
        structs = [
            ( 3,  7, 4,      numeric, "1"),
            ( 7,  8, 1,      numeric, "3"),
            (13, 14, 1, alphaNumeric, "R"),
            (15, 17, 2,      numeric, "01"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setCancel(self):
        structs = [
            (15, 17, 2, numeric, "02"),                                               # Indica Baixa do boleto
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0, 3, 3, numeric, bank.bankId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index)                                                # Indica index do lote
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setFine(self, date, fine):
        code = "0" if int(fine) == 0 else "2"
        structs = [
            (65,  66,  1, numeric, code),                                        # 0-Sem multa/ 1-Valor em reais/ 2-Porcentagem
            (66,  74,  8, numeric, date),
            (74,  89, 15, numeric, fine)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)




# coding: utf-8

from ....row import Row
from ....characterType import numeric


class TrailerLot:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def defaultValues(self):
        structs = [
            ( 3,   7,  4, numeric, "1"),
            ( 7,   8,  1, numeric, "5"),
            (41,  59, 18, numeric, "0"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setLotNumberOfRegisters(self, num):
        structs = [
            (17,  23,  6, numeric, num),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSumOfValues(self, sum):
        structs = [
            (23, 41, 18, numeric, sum),                     # Soma dos valores dos lotes
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSumOfValuesNonBarCodeTax(self, sum, otherSum, additionSum, totalSum):
        structs = [
            (23, 37, 14, numeric, sum),                     # Soma dos valores principais dos lotes
            (37, 51, 14, numeric, otherSum),                # Soma dos valores outras entidades dos lotes
            (51, 65, 14, numeric, additionSum),             # Soma dos valores acréscimos dos lotes
            (65, 79, 14, numeric, totalSum),                # Soma dos valores totais dos lotes
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0, 3, 3, numeric, bank.bankId),                   # Código do banco debitado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (3, 7, 4, numeric, index)                          # Indica index do lote
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from .libs.middlewares.row import validateFormatter
from .characterType import numeric, alphaNumeric


class Row:

    @classmethod
    def setStructs(cls, structs, content):
        for (start, end, len, type, value) in structs:
            replacement = cls.__formatted(
                string=str(value),
                charactersType=type,
                numberOfCharacters=len,
                defaultCharacter={numeric: "0", alphaNumeric: " "}[type]
            )
            content = content[:start] + replacement + content[start+len:]
        return content

    @classmethod
    @validateFormatter
    def __formatted(cls, string, charactersType, numberOfCharacters, defaultCharacter=" "):
        """
            This method fix the received String and a default complement according the alignment
            and cut the string if it' bigger than number of characters

            Args:
                string:             String to be completed
                charactersType:     Can be .numeric or .alphaNumeric
                numberOfCharacters: Integer that represents the max string len
                defaultCharacter:   Single string with default character to be completed if string is short
            Returns:
                String formatted
        """
        if type(string) != str:              return defaultCharacter * numberOfCharacters
        if len(string) > numberOfCharacters: return string[:numberOfCharacters]
        if charactersType == numeric:        return defaultCharacter * (numberOfCharacters - len(string)) + string
        if charactersType == alphaNumeric:   return string + defaultCharacter * (numberOfCharacters - len(string))
        return string


# coding: utf-8

from ....row import Row
from ....characterType import numeric, alphaNumeric


class SegmentJ52:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def defaultValues(self):
        structs = [
            ( 3,   7,  4,      numeric, "1"),     # Lote do Registro
            ( 7,   8,  1,      numeric, "3"),     # Tipo de Registro
            (13,  14,  1, alphaNumeric, "J"),     # Código de Segmento
            (14,  17,  3,      numeric, "0"),     # Tipo de Movimento
            (17,  19,  2,      numeric, "52"),
            (75,  91,  16,     numeric, "0"),
            (131, 147, 16,     numeric, "0"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            ( 0,  3, 3, numeric, bank.bankId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSender(self, user):
        structs = [
            (19, 20,  1,     numeric,  "1" if len(user.identifier) == 11 else "2"),
            (20, 35, 15,     numeric,  user.identifier),
            (35, 75, 40, alphaNumeric, user.name)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setReceiverTaxId(self, receiverTaxId):
        structs = [
            (75, 76,  1,     numeric,  "1" if len(receiverTaxId) == 11 else "2"),
            (76, 91, 15,     numeric,  receiverTaxId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setLot(self, lot):
        structs = [
            (3,  7, 4, numeric, lot),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from .....cnab240.libs.paymentType import NonBarcodeTaxPayment, BarcodePayment
from .occurrences import occurrences


class PaymentResponseStatus:

    success = "success"
    failed = "failed"
    scheduled = "scheduled"
    unknown = "unknown"


class PaymentType:

    transfer = "transfer"
    chargePayment = "charge-payment"
    nonBarcodeTaxPayment = "tax-payment"
    barcodePayment = "barcode-payment"


class PaymentResponse:

    def __init__(self, identifier=None, occurrences=None, content=None, authentication=None, amountInCents=None, paymentType=None, nonBarcodeTax=None):
        self.identifier = identifier
        self.occurrences = occurrences
        self.content = content or []
        self.authentication = authentication
        self.amountInCents = amountInCents
        self.type = paymentType
        self.nonBarcodeTax = nonBarcodeTax

    def occurrencesText(self):
        return [occurrences[occurrenceId] for occurrenceId in self.occurrences]

    def occurrencesTextAtIndex(self, index):
        occurrenceId = self.occurrences[index]
        return occurrences[occurrenceId]

    def status(self):
        if "00" in self.occurrences:
            return PaymentResponseStatus.success
        if "BD" in self.occurrences:
            return PaymentResponseStatus.scheduled
        if [code in self.occurrences for code in ["RJ", "DV"]].count(True) > 0:
            return PaymentResponseStatus.failed
        return PaymentResponseStatus.unknown

    def contentText(self, breakLine="\n"):
        return breakLine.join(self.content)


class PaymentParser:

    @classmethod
    def parseFile(cls, file):
        lines = file.readlines()
        return cls.parseLines(lines)

    @classmethod
    def parseText(cls, text):
        lines = text.splitlines()[:-1]
        return cls.parseLines(lines)

    @classmethod
    def parseLines(cls, lines):
        result = []
        currentResponse = None
        for line in lines:
            if line[7] in ["0", "1", "9"]:
                continue

            if line[7] == "3" and line[13] in ["A", "J", "O", "N"]:
                if currentResponse is not None:
                    result.append(currentResponse)
                currentResponse = PaymentResponse()

            if line[7] == "3" and line[13] == "A":
                currentResponse.content.append(line)
                currentResponse.identifier = cls._getIdentifierSegmentA(line)
                currentResponse.occurrences = cls._getOccurrences(line)
                currentResponse.amountInCents = cls._getAmountSegmentA(line)
                currentResponse.type = PaymentType.transfer
            elif line[7] == "3" and line[13] == "J":
                currentResponse.content.append(line)
                currentResponse.identifier = cls._getIdentifierSegmentJ(line)
                currentResponse.occurrences = cls._getOccurrences(line)
                currentResponse.amountInCents = cls._getAmountSegmentJ(line)
                currentResponse.type = PaymentType.chargePayment
            elif line[7] == "3" and line[13] == "O":
                currentResponse.content.append(line)
                currentResponse.identifier = cls._getIdentifierSegmentO(line)
                currentResponse.occurrences = cls._getOccurrences(line)
                currentResponse.amountInCents = cls._getAmountSegmentO(line)
                currentResponse.type = PaymentType.barcodePayment
            elif line[7] == "3" and line[13] == "N":
                currentResponse.content.append(line)
                currentResponse.identifier = cls._getIdentifierSegmentN(line)
                currentResponse.occurrences = cls._getOccurrences(line)
                currentResponse.nonBarcodeTax = cls._getNonBarcodeTaxSegmentN(line)
                currentResponse.type = (
                    PaymentType.barcodePayment
                    if currentResponse.nonBarcodeTax == BarcodePayment.fgts
                    else PaymentType.nonBarcodeTaxPayment
                )
            elif line[7] == "3" and line[13] == "Z":
                currentResponse.content.append(line)
                currentResponse.authentication = cls._getAuthentication(line)

            if line[7] == "5" and currentResponse is not None:
                result.append(currentResponse)
                currentResponse = None

        return result

    @classmethod
    def _getOccurrences(cls, line):
        occurrencesString = line[230:240].strip()
        return cls._splitString(occurrencesString)

    @classmethod
    def _splitString(cls, string):
        return [string[i:i+2] for i in range(0, len(string), 2)]

    @classmethod
    def _getAmountSegmentA(cls, line):
        return int(line[119:134].strip())

    @classmethod
    def _getAmountSegmentJ(cls, line):
        return int(line[152:167].strip())

    @classmethod
    def _getAmountSegmentO(cls, line):
        return int(line[144:159].strip())

    @classmethod
    def _getIdentifierSegmentA(self, line):
        return line[73:93].strip()

    @classmethod
    def _getIdentifierSegmentJ(self, line):
        return line[182:202].strip()

    @classmethod
    def _getIdentifierSegmentO(self, line):
        return line[174:194].strip()

    @classmethod
    def _getIdentifierSegmentN(self, line):
        return line[195:215].strip()

    @classmethod
    def _getAuthentication(cls, line):
        return line[14:78].strip()

    @classmethod
    def _getNonBarcodeTaxSegmentN(self, line):
        taxTypeId = line[17:19].strip()
        return {
            "01": NonBarcodeTaxPayment.gps,
            "02": NonBarcodeTaxPayment.darf,
            "11": BarcodePayment.fgts,
        }[taxTypeId]





class DAC:

    @classmethod
    def calculate(cls, branch, accountNumber, wallet, bankNumber):
        x1 = branch + accountNumber + wallet + bankNumber
        x2 = "12" * (len(x1) / 2) + ("1" if len(x1) % 2 == 1 else "")
        out = [int(a) * int(b) for (a, b) in zip(x1, x2)]
        s = sum(map(lambda x: x/10 + x % 10, out))
        return (10 - s % 10) % 10

    @classmethod
    def calculateTaxDac(cls, productId, segmentId, currency, amount, company, freeField):
        x1 = productId + segmentId + currency + amount + company + freeField
        x2 = "12" * (len(x1) / 2) + ("1" if len(x1) % 2 == 1 else "")
        out = [int(a) * int(b) for (a, b) in zip(x1, x2)]
        s = sum(map(lambda x: x/10 + x % 10, out))
        return (10 - s % 10) % 10



from ....row import Row
from ....characterType import numeric


class TrailerLot:

    def __init__(self):
        self.content = "00000000         0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000                                                                                                                     "
        self.defaultValues()

    def defaultValues(self):
        structs = [
            (3, 7, 4, numeric, "1"),
            (7, 8, 1, numeric, "5"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setNumberOfLotsAndRegisters(self, num):
        structs = [
            (17, 23, 6, numeric, 2 + 3*num),
            (23, 29, 6, numeric, num),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0,   3,  3, numeric, bank.bankId),    # Codigo do banco debitado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSumOfValues(self, sum):
        structs = [
            (29, 46, 17, numeric, sum)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)


# coding: utf-8

from ....characterType import numeric, alphaNumeric
from ....row import Row


class SegmentA:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def amountInCents(self):
        return int(self.content[119:134])

    def defaultValues(self):
        structs = [
            (  3,   7,  4,      numeric, "1"),
            (  7,   8,  1,      numeric, "3"),
            ( 13,  14,  1, alphaNumeric, "A"),
            ( 14,  20,  6,      numeric, "0"),
            (101, 104,  3, alphaNumeric, "REA"),
            (104, 119, 15,      numeric, "0"),
            (154, 177, 23,      numeric, "0"),
            (197, 203,  6,      numeric, "0"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0,   3,  3, numeric, bank.bankId),              # Código do banco debitado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setReceiver(self, user):
        structs = [
            ( 43,  73, 30,  alphaNumeric, user.name),        # Nome FAvorecido
            (203, 217, 14,       numeric, user.identifier),  # CPF/CNPJ Favorecido
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setReceiverBank(self, bank):
        structs = [
            (20, 23,  3,  numeric, bank.bankId),             # Código do banco do Favorecido
            (23, 28,  5,  numeric, bank.branchCode),         # Agencia Favorecido
            (29, 41, 12,  numeric, bank.accountNumber),      # Conta Favorecido
            (42, 43,  1,  numeric, bank.accountVerifier),    # DAC FAvorecido
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setAmountInCents(self, amount):
        structs = [
            (119, 134, 15, numeric, amount)                  # Valor ao Favorecido
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index),                       # Indica index do lote
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setLot(self, lot):
        structs = [
            (3,  7, 4, numeric, lot),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setScheduleDate(self, date):
        structs = [
            (93, 101, 8, numeric, date)                      # Data para transferencia
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setInfo(self, reason):
        structs = [
            (217, 219, 2, alphaNumeric, reason)              # Motivo da transferencia
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setIdentifier(self, identifier):
        structs = [
            (73, 93, 20, alphaNumeric, identifier)           # Id da empresa na transação
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from febraban.cnab240.libs.enum import Enum


class PaymentMethod(Enum):
    # NOTA 5 (Page 38)
    utility = "13"
    darf = "16"
    gps = "17"
    das = "18"
    municipal = "19"
    darj = "21"
    gare = "22"
    ipva = "25"
    dpvat = "27"
    itau = "30"
    other = "31"
    receipt = "32"
    fgts = "35"
    tedOther = "41"
    tedSame = "43"
    salaryCard = "60"
    barcode = "91"

    @classmethod
    def taxes(cls):
        return [cls.darf, cls.gps, cls.das, cls.municipal, cls.darj, cls.gare, cls.ipva, cls.dpvat, cls.fgts]

    @classmethod
    def nonBarcodeTaxes(cls):
        return [cls.darf, cls.gps, cls.das, cls.darj, cls.gare, cls.ipva, cls.dpvat, cls.fgts]






class User:

    def __init__(self, name, identifier, bank=None, address=None):
        self.name = name
        self.identifier = identifier
        self.bank = bank
        self.address = address


class UserBank:

    def __init__(self, bankId, branchCode, accountNumber, accountVerifier, bankName=""):
        self.bankId = bankId
        self.bankName = bankName
        self.accountNumber = accountNumber
        self.branchCode = branchCode
        self.accountVerifier = accountVerifier


class UserAddress:

    def __init__(self, streetLine1, city, stateCode, zipCode, district="", streetLine2=""):
        self.streetLine1 = streetLine1
        self.streetLine2 = streetLine2
        self.district = district
        self.city = city
        self.stateCode = stateCode
        self.zipCode = zipCode


# coding: utf-8
# Pagamento de Contas de Concessionárias e Tributos com código de barras # Página 28
from ....row import Row
from ....characterType import numeric, alphaNumeric


class SegmentO:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def defaultValues(self):
        structs = [
            (  3,   7,  4,      numeric,   "1"),
            (  7,   8,  1,      numeric,   "3"),  # Tipo de Registro
            ( 13,  14,  1, alphaNumeric,   "O"),  # Código de Segmento
            ( 14,  17,  3,      numeric,   "0"),  # Tipo de Movimento
            (103, 106,  3, alphaNumeric, "REA"),  # Tipo da moeda
            (144, 159, 15,      numeric,   "0"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0, 3, 3, numeric, bank.bankId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setScheduleDate(self, date):
        structs = [
            (136, 144,  8, numeric, date)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setLot(self, lot):
        structs = [
            (3,  7, 4, numeric, lot),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setDealerName(self, dealerName):                    # NOME DA CONCESSIONÁRIA / CONTRIBUINTE
        structs = [
            (65, 95, 30, alphaNumeric, dealerName),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setDueDate(self, dueDate):
        structs = [
            (95, 103, 8, numeric, dueDate),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setLineNumber(self, lineNumber):
        structs = [
            (17,   65, 48, numeric, lineNumber.number),
            (121, 136, 15, numeric, lineNumber.amount),  # Valor a pagar
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setBankIdentifier(self, identifier, dac):
        structs = [
            (40, 48, 8, numeric, identifier),                # Numero dado pelo Banco
            (48, 49, 1, numeric, dac)                        # Verificador do numero dado pelo Banco
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setIdentifier(self, identifier):
        structs = [
            (174, 194, 20, alphaNumeric, identifier),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def ourNumber(self, ourNumber):                           # Nosso número: Nota 12
        structs = [                                           # Somente para cancelamento ou alteração
            (215, 230, 15, alphaNumeric, ourNumber),          # Retornado pelo Itau a cada pagamento
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setOccurences(self, occurrenceId):                    # Nota 8
        structs = [
            (230, 240, 10, alphaNumeric, occurrenceId)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from .headerLot import HeaderLot
from .trailerLot import TrailerLot
from ....itau.sispag import Transfer, ChargePayment, BarCodePayment, NonBarCodePayment
from ....libs.paymentKind import PaymentKind
from ....libs.paymentMethod import PaymentMethod


class Lot:

    def __init__(self):
        self.headerLot = HeaderLot()
        self.registers = []
        self.trailerLot = TrailerLot()
        self.kind = ""
        self.method = ""
        self.amount = 0
        self.otherAmount = 0
        self.additionAmount = 0
        self.totalAmount = 0
        self.index = 1
        self.count = 0

    def add(self, register):
        register.setPositionInLot(index=self.index)
        self.registers.append(register)
        self.amount += register.amountInCents()
        if self._isBoletoPayment():
            self.totalAmount += register.totalAmountInCents()
        if self._isNonBarCodeTax():
            self.otherAmount += register.otherAmountInCents()
            self.additionAmount += register.additionAmountInCents()
            self.totalAmount += register.totalAmountInCents()
        self.index += 1

    def setLotNumber(self, index):
        self.headerLot.setPositionInLot(index)
        self.trailerLot.setPositionInLot(index)
        for register in self.registers:
            register.setLot(index)

    def setSender(self, sender):
        self.headerLot.setSender(sender)
        self.headerLot.setSenderBank(sender.bank)
        self.headerLot.setSenderAddress(sender.address)
        self.trailerLot.setSenderBank(sender.bank)

    def setHeaderLotType(self, kind=PaymentKind.vendor, method=PaymentMethod.tedOther):
        """
        Trasfers:
            kind:   String - Kind of payment - 20 Fornecedores, read: NOTES 4
            method: String - Payment method  - 41 TED Outro titular, 43 TED Mesmo titular, 01 ITAU account. read: NOTES 5

        Charge-payments:
            kind:   String - Kind of payment - 98 Diversos, read: NOTES 4
            method: String - Payment method  - 30 Pagamento Boleto Itau, 31 Pagamento Boleto outros Bancos. read: NOTES 5

        Utilities:
            kind:   String - Kind of payment - 98 Diversos, read: NOTES 4
            method: String - Payment method  - 13 Concessionarias. read: NOTES 5

        Tax-payments:
            kind:   String - Kind of payment - 22 Tributos, read: NOTES 4
            method: String - Payment method  - 91 GNRE e Tributos com Codigo de Barras,
                                               19 IPTU/ISS/Outros Tributos Municipais. read: NOTES 5,
                                               16 DARF (No barcode)

        """
        self.kind = kind
        self.method = method
        self.headerLot.setInfo(kind, method)

    def toString(self):
        self.count = (2 + self._count(Transfer) + 2 * self._count(ChargePayment)
                      + self._count(BarCodePayment) + self._count(NonBarCodePayment))
        self.trailerLot.setLotNumberOfRegisters(
            num=self.count
        )

        if self._isNonBarCodeTax():
            self.trailerLot.setSumOfValuesNonBarCodeTax(
                sum=self.amount,
                otherSum=self.otherAmount,
                additionSum=self.additionAmount,
                totalSum=self.totalAmount,
            )
        elif self._isBoletoPayment():
            self.trailerLot.setSumOfValues(sum=self.totalAmount)

        else:
            self.trailerLot.setSumOfValues(sum=self.amount)

        registersToString = "\r\n".join([register.toString() for register in self.registers])
        return "%s\r\n%s\r\n%s" % (
            self.headerLot.content,
            registersToString,
            self.trailerLot.content,
        )

    def _count(self, cls):
        return len([register for register in self.registers if isinstance(register, cls)])

    def _isBoletoPayment(self):
        return self.kind == PaymentKind.diverse and self.method in [PaymentMethod.itau, PaymentMethod.other]

    def _isNonBarCodeTax(self):
        return self.kind == PaymentKind.tribute and self.method in PaymentMethod.nonBarcodeTaxes()





from febraban.cnab240.libs.enum import Enum


class PaymentKind(Enum):

    # NOTA 4 (Page 38)
    dividend = "15"
    vendor = "20"
    tribute = "22"
    salary = "30"
    investmentFund = "40"
    insurance = "50"
    travel = "60"
    representative = "80"
    benefits = "90"
    diverse = "98"






# coding: utf-8

from ....row import Row
from ....characterType import numeric, alphaNumeric


class Header:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def defaultValues(self):
        structs = [
            (   3,   8, 5, numeric, "0"),                          # Tipo de registro
            (  14,  17, 3, numeric, "81"),                         # Versão do Layout
            ( 142, 143, 1, numeric, "1"),                          # 1 - Remessa / 2 - Retorno
            ( 157, 166, 9, numeric, "0"),
            ( 166, 171, 5, numeric, "0"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setGeneratedFileDate(self, datetime):
        structs = [
            (143, 151, 8, numeric, datetime.strftime("%d%m%Y")),   # Dia que o arquivo foi gerado
            (151, 157, 6, numeric, datetime.strftime("%H%M%S")),   # Horario que o arquivo foi gerado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSender(self, user):
        structs = [
            (17,  18,  1,      numeric, "1" if len(user.identifier) == 11 else "2"),
            (18,  32, 14,      numeric, user.identifier),
            (72, 102, 30, alphaNumeric, user.name)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            ( 0,   3,  3, numeric, bank.bankId),
            (52,  57,  5, numeric, bank.branchCode),
            (58,  70, 12, numeric, bank.accountNumber),
            (71,  72,  1, numeric, bank.accountVerifier),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from datetime import date, timedelta


class BarCodeJ:

    def __init__(self, number):
        self.number = number
        self.bankId = self.number[0:3]
        self.currency = self.number[3:4]
        self.dac = self.number[4:5]
        self.dueFactor = self.number[5:9]
        self.amount = self.number[9:19]
        self.freeField = self.number[19:44]
        self.baseDate = date(year=1997, month=10, day=7)
        self.dueDate = self.baseDate + timedelta(days=int(self.dueFactor))


class BarCodeN:

    def __init__(self, number):
        self.number = number
        self.productId = self.number[0]
        self.segmentId = self.number[1]
        self.currency = self.number[2]
        self.dac = self.number[3]
        self.amount = self.number[4:15]
        self.companyId = self.number[15:19]
        self.freeField = self.number[19:]


class LineNumberO:

    def __init__(self, number):
        self.number = number
        self.productId = self.number[0]
        self.segmentId = self.number[1]
        self.currency = self.number[2]
        self.dac = self.number[3]
        self.dv1 = self.number[11]
        self.amount = self.number[4:11] + self.number[12:16]
        self.companyId = self.number[16:20]
        self.dv2 = self.number[23]
        self.dv3 = self.number[35]
        self.dv4 = self.number[47]
        self.freeField = self.number[20:23] + self.number[24:35] + self.number[36:47]



# coding: utf-8
from ....row import Row
from ....characterType import numeric, alphaNumeric


class SegmentJ:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def defaultValues(self):
        structs = [
            (  3,   7,  4,      numeric,   "1"),         # Lote do Registro
            (  7,   8,  1,      numeric,   "3"),         # Tipo de Registro
            ( 13,  14,  1, alphaNumeric,   "J"),         # Código de Segmento
            ( 14,  17,  3,      numeric,   "0"),         # Tipo de Movimento
            (114, 144, 30,      numeric,   "0"),
            (167, 182, 15,      numeric,   "0"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0, 3, 3, numeric, bank.bankId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setBarCode(self, barCode):
        structs = [
            ( 17,  20,  3, numeric, barCode.bankId),
            ( 20,  21,  1, numeric, barCode.currency),
            ( 21,  22,  1, numeric, barCode.dac),
            ( 22,  26,  4, numeric, barCode.dueFactor),
            ( 26,  36, 10, numeric, barCode.amount),
            ( 36,  61, 25, numeric, barCode.freeField),
            ( 99, 114, 15, numeric, barCode.amount),                        # Valor Nominal do Título
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setAmounts(self, discountAmount, addedAmount, totalAmount):
        structs = [
            (114, 129, 15, numeric, discountAmount),                        # Valor do Desconto
            (129, 144, 15, numeric, addedAmount),                           # Valor do Acréscimo
            (152, 167, 15, numeric, totalAmount),                           # Valor do Pagamento
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setScheduleDate(self, date):
        structs = [
            (144, 152,  8, numeric, date)
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setDueDate(self, dueDate):
        structs = [
            ( 91,  99,  8, numeric, dueDate),    # Data de Vencimento
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setIdentifier(self, identifier):
        structs = [
            (182, 202, 20, alphaNumeric, identifier),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setLot(self, lot):
        structs = [
            (3,  7, 4, numeric, lot),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from .occurrences import occurrences
from .errors import errors03, errors17, errors15, errors16, errors18


class SlipResponseStatus:

    registered = "registered"
    paid = "paid"
    canceled = "canceled"
    overdue = "overdue"
    failed = "failed"
    unknown = "unknown"


class SlipResponse:

    def __init__(self, identifier=None, occurrence=None, content=None, amountInCents=None, fine=None, discount=None, errors=None):
        self.identifier = identifier
        self.occurrence = occurrence
        self.amountInCents = amountInCents
        self.fine = fine
        self.discount = discount
        self.content = content or []
        self.errors = errors or []

    def occurrenceText(self):
        return occurrences[self.occurrence]

    def status(self):
        if self.occurrence == "02":
            return SlipResponseStatus.registered
        if self.occurrence in ["03", "15", "16", "17", "18"]:
            return SlipResponseStatus.failed
        if self.occurrence in ["06", "08"]:
            return SlipResponseStatus.paid
        if self.occurrence == "09":
            return SlipResponseStatus.canceled
        return SlipResponseStatus.unknown

    def contentText(self, breakLine="\n"):
        return breakLine.join(self.content)

    def failureErrors(self):
        errorDict = {
            "03": errors03,
            "15": errors15,
            "16": errors16,
            "17": errors17,
            "18": errors18,
        }.get(self.occurrence) or {}

        return [errorDict[errorCode] for errorCode in self.errors if errorCode in errorDict.keys()]

    def amountPaid(self):
        return self.amountInCents + self.fine - self.discount


class SlipParser:

    @classmethod
    def parseFile(cls, file):
        lines = file.readlines()
        return cls.parseLines(lines)

    @classmethod
    def parseText(cls, text):
        lines = text.splitlines()[:-1]
        return cls.parseLines(lines)

    @classmethod
    def parseLines(cls, lines):
        result = []
        for line in lines:
            if line[7] == "3" and line[13] == "T":
                currentResponse = SlipResponse()
                currentResponse.content.append(line)
                currentResponse.amountInCents = int(line[81:96])
                currentResponse.occurrence = line[15:17]
                currentResponse.identifier = line[105:130].strip()
                currentResponse.errors = [line[213 + i:215 + i] for i in range(0, 8, 2) if line[i:i + 2] != "  "]
            elif line[7] == "3" and line[13] == "U":
                currentResponse.content.append(line)
                currentResponse.fine = int(line[17:32])
                currentResponse.discount = int(line[32:47])
                result.append(currentResponse)
            elif line[7] == "3" and line[13] == "P":
                currentResponse = SlipResponse()
                currentResponse.content.append(line)
                currentResponse.amountInCents = int(line[85:100])
                currentResponse.occurrences = [line[15:17]]
                currentResponse.identifier = line[195:220].strip()
            elif line[7] == "3" and line[13] == "Q":
                currentResponse.content.append(line)
                result.append(currentResponse)
        return result



# coding: utf-8
# Pagamento de Contas de Concessionárias e Tributos sem código de barras # Página 31
from ....row import Row
from ....characterType import numeric, alphaNumeric


class SegmentN:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def defaultValues(self):
        structs = [
            (  3,   7,  4,      numeric,   "1"),
            (  7,   8,  1,      numeric,   "3"),  # Tipo de Registro
            ( 13,  14,  1, alphaNumeric,   "N"),  # Código de Segmento
            ( 14,  17,  3,      numeric,   "0"),  # Tipo de Movimento
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0, 3, 3, numeric, bank.bankId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setPositionInLot(self, index):
        structs = [
            (8, 13, 5, numeric, index),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setLot(self, lot):
        structs = [
            (3,  7, 4, numeric, lot),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setTaxPaymentIdentifier(self, id):
        structs = [
            (17, 19, 2, numeric, id),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setRevenueCode(self, code):
        structs = [
            (19, 23, 4, numeric, code),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setTaxIdType(self, taxIdType):
        structs = [
            (23, 24, 1, numeric, taxIdType),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setTaxId(self, taxId):
        structs = [
            (24, 38, 14, alphaNumeric, taxId),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setReferenceDate(self, referenceDate):
        structs = [
            (38, 46, 8, numeric, referenceDate),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setReferenceNumber(self, referenceNumber):
        structs = [
            (46, 63, 17, numeric, referenceNumber),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setNominalAmount(self, dueDate):
        structs = [
            (63, 77, 14, numeric, dueDate),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setFineAmount(self, fineAmount):
        structs = [
            (77, 91, 14, numeric, fineAmount),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setInterestAmount(self, interestAmount):
        structs = [
            (91, 105, 14, numeric, interestAmount),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setTotalAmount(self, totalAmount):
        structs = [
            (105, 119, 14, numeric, totalAmount),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setDueDate(self, dueDate):
        structs = [
            (119, 127, 8, numeric, dueDate),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setScheduleDate(self, paymentDate):
        structs = [
            (127, 135, 8, numeric, paymentDate),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setFgtsScheduleDate(self, paymentDate):
        structs = [
            (143, 151, 8, numeric, paymentDate),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setIdentifier(self, identifier):
        structs = [
            (195, 215, 20, alphaNumeric, identifier),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setBarCode(self, barCode):
        structs = [
            (38, 86, 48, alphaNumeric, barCode),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setFgtsAmount(self, amount):
        structs = [
            (151, 165, 14, numeric, amount),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setFgtsNullValues(self):
        structs = [
            (86, 102, 16, numeric, 0),
            (102, 111, 9, numeric, 0),
            (111, 113, 2, numeric, 0),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from os import path
from setuptools import setup, find_packages

with open(path.join(path.dirname(__file__), "README.srt")) as readme:
    README = readme.read()

setup(
    name="febraban",
    packages=find_packages(),
    include_package_data=True,
    description="A library to generate files in FEBRABAN formats",
    long_description=README,
    license="MIT License",
    url='https://github.com/starkbank/febraban.git',
    author="Stark Bank",
    author_email="developers@starkbank.com",
    keywords=["febraban", "cnab", "cnab 240", "cnab240", "febraban240", "transfer", "billing", "bank"],
    version="0.9.1"
)

"""
Deployment instructions
-----------------------

Global
~~~~~~

Before deployment, change the project *version* on ``setup.py`` file and
set the correct *download*\ url\_. Then make sure you have the file
``~/.pypirc`` with the content below and the correct credentials instead
of the provided placeholders:

::

    [distutils]
    index-servers =
        pypi
        pypitest

    [pypi]
    repository:https://upload.pypi.org/legacy/
    username:myusername
    password:mypassword

    [pypitest]
    repository:https://test.pypi.org/legacy/
    username:myusername
    password:mypassword

### Create a source distribution:

Run ```python setup.py sdist``` inside the project directory.

### Install twine:

```pip install twine```

### Upload package to pypi:

```twine upload dist/*```


Live environment
~~~~~~~~~~~~~~~~

For unit test:

```
python -m unittest discover -s febraban/cnab240/tests -p '*Test.py'
```

For release a new version:

```
python setup.py sdist upload -r pypi`` inside the project
```

"""


from .segmentA import SegmentA


class Transfer:

    def __init__(self):
        self.segmentA = SegmentA()

    def toString(self):
        return self.segmentA.content

    def amountInCents(self):
        return self.segmentA.amountInCents()

    def setSender(self, user):
        self.segmentA.setSenderBank(user.bank)

    def setReceiver(self, user):
        self.segmentA.setReceiver(user)
        self.segmentA.setReceiverBank(user.bank)

    def setAmountInCents(self, value):
        self.segmentA.setAmountInCents(value)

    def setPositionInLot(self, index):
        self.segmentA.setPositionInLot(index)

    def setLot(self, lot):
        self.segmentA.setLot(lot)

    def setScheduleDate(self, date):
        self.segmentA.setScheduleDate(date)

    def setInfo(self, reason="10"):
        """
        This method set config information in the payment

        Args:
            reason: String - Payment reason  - 10 Credito em Conta Corrente, read: NOTES 26
        """
        self.segmentA.setInfo(reason)

    def setIdentifier(self, identifier):
        self.segmentA.setIdentifier(identifier)



from .segmentJ import SegmentJ
from .segmentJ52 import SegmentJ52


class ChargePayment:

    def __init__(self):
        self.segmentJ = SegmentJ()
        self.segmentJ52 = SegmentJ52()
        self.amount = 0
        self.discountAmount = 0
        self.additionAmount = 0
        self.totalAmount = 0

    def setPayment(self, **kwargs):
        barCode = kwargs["barCode"]
        dueDate = kwargs.get("dueDate") or barCode.dueDate
        self.setSender(kwargs["sender"])
        self.setReceiverTaxId(kwargs["receiverTaxId"])
        self.setBarCode(barCode)
        self.setIdentifier(kwargs["identifier"])
        self.setScheduleDate(kwargs["scheduleDate"].strftime("%d%m%Y"))
        self.setDueDate(dueDate.strftime("%d%m%Y"))
        self.setAmounts(
            discountAmount=kwargs["discountAmount"],
            addedAmount=kwargs["addedAmount"],
            totalAmount=kwargs["totalAmount"],
        )

    def toString(self):
        return "\r\n".join((
            self.segmentJ.content,
            self.segmentJ52.content,
        ))

    def amountInCents(self):
        return self.amount

    def totalAmountInCents(self):
        return self.totalAmount

    def setSender(self, user):
        """Sets the sender for the payment. The sender represents a user, its bank and its address."""
        self.segmentJ.setSenderBank(user.bank)
        self.segmentJ52.setSender(user)
        self.segmentJ52.setSenderBank(user.bank)

    def setReceiverTaxId(self, receiverTaxId):
        """Sets the receiver for the payment. Only the receiver's taxId is necessary."""
        self.segmentJ52.setReceiverTaxId(receiverTaxId)

    def setIdentifier(self, identifier):
        """Sets the charge identifier that will be returned from the bank. Used for matching results."""
        self.segmentJ.setIdentifier(identifier)

    def setScheduleDate(self, paymentDate):
        """Sets the payment date to be sent to the bank."""
        self.segmentJ.setScheduleDate(paymentDate)

    def setDueDate(self, dueDate):
        """Sets the payment due date extracted from either CIP or the barcode itself"""
        self.segmentJ.setDueDate(dueDate)

    def setBarCode(self, barCode):
        self.segmentJ.setBarCode(barCode)
        self.amount = int(barCode.amount)

    def setAmounts(self, discountAmount, addedAmount, totalAmount):
        self.segmentJ.setAmounts(
            discountAmount=discountAmount,
            addedAmount=addedAmount,
            totalAmount=totalAmount
        )
        self.discountAmount = discountAmount
        self.additionAmount = addedAmount
        self.totalAmount = totalAmount

    def setPositionInLot(self, index):
        index = 2 * index - 1
        self.segmentJ.setPositionInLot(index)
        self.segmentJ52.setPositionInLot(index + 1)

    def setLot(self, lot):
        self.segmentJ.setLot(lot)
        self.segmentJ52.setLot(lot)



# coding: utf-8

from ....row import Row
from ....characterType import numeric


class Trailer:

    def __init__(self):
        self.content = " " * 240
        self.defaultValues()

    def defaultValues(self):
        structs = [
            ( 3,  8, 5, numeric, "99999"),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setNumberOfLotsAndRegisters(self, sum, num):
        structs = [
            (17, 23, 6, numeric, num),
            (23, 29, 6, numeric, sum),
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)

    def setSenderBank(self, bank):
        structs = [
            (0,   3,  3, numeric, bank.bankId),              # Código do banco debitado
        ]
        self.content = Row.setStructs(structs=structs, content=self.content)



from febraban.cnab240.itau.sispag.payment.segmentN import SegmentN


class NonBarCodePayment:
    def __init__(self):
        self.segmentN = SegmentN()
        self.amount = 0
        self.otherAmount = 0
        self.additionAmount = 0
        self.totalAmount = 0

    def toString(self):
        return "\r\n".join((
            self.segmentN.content,
        ))

    def amountInCents(self):
        return self.amount

    def otherAmountInCents(self):
        return self.otherAmount

    def additionAmountInCents(self):
        return self.additionAmount

    def totalAmountInCents(self):
        return self.totalAmount

    def setIdentifier(self, identifier):
        self.segmentN.setIdentifier(identifier)

    def setSender(self, user):
        self.segmentN.setSenderBank(user.bank)

    def setPositionInLot(self, index):
        self.segmentN.setPositionInLot(index)

    def setLot(self, lot):
        self.segmentN.setLot(lot)

